@@ -20,16 +20,21 @@ import urlparse
 import luigi.format
 import datetime
 import re
+from luigi.target import FileSystem, FileSystemTarget, FileAlreadyExists
+import configuration
 
 
 class HDFSCliError(Exception):
     def __init__(self, command, returncode, stdout, stderr):
+        self.returncode = returncode
+        self.stdout = stdout
+        self.stderr = stderr
         msg = ("Command %r failed [exit code %d]\n" +
-                "---stdout---\n" +
-                "%s\n" +
-                "---stderr---\n" +
-                "%s" +
-                "------------") % (command, returncode, stdout, stderr)
+               "---stdout---\n" +
+               "%s\n" +
+               "---stderr---\n" +
+               "%s" +
+               "------------") % (command, returncode, stdout, stderr)
         super(HDFSCliError, self).__init__(msg)
 
 
@@ -40,37 +45,34 @@ def call_check(command):
         raise HDFSCliError(command, p.returncode, stdout, stderr)
     return stdout
 
-def use_cdh4_syntax():
+
+def get_hdfs_syntax():
     """
     CDH4 (hadoop 2+) has a slightly different syntax for interacting with
     hdfs via the command line. The default version is CDH4, but one can
-    override this setting with "cdh3" in the hadoop section of the config in
+    override this setting with "cdh3" or "apache1" in the hadoop section of the config in
     order to use the old syntax
     """
-    import interface
-    return interface.get_config().get("hadoop", "version", "cdh4").lower() == "cdh4"
+    return configuration.get_config().get("hadoop", "version", "cdh4").lower()
 
 
 def tmppath(path=None):
     return tempfile.gettempdir() + '/' + (path + "-" if path else "") + "luigitemp-%08d" % random.randrange(1e9)
 
 
-class HdfsClient(object):
+class HdfsClient(FileSystem):
+    """This client uses Apache 2.x syntax for file system commands, which also matched CDH4"""
     def exists(self, path):
-        """ Use `hadoop fs -ls -d` to check file existence
-
-        `hadoop fs -test -e` can't be (reliably) used at this time since there
-        is no good way of distinguishing file non-existence of files
-        from errors when running the comman (same return code)
+        """ Use `hadoop fs -stat to check file existance
         """
 
-        cmd = ['hadoop', 'fs', '-ls', '-d', path]
+        cmd = ['hadoop', 'fs', '-stat', path]
         p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         stdout, stderr = p.communicate()
         if p.returncode == 0:
             return True
         else:
-            not_found_pattern = "^ls: `.*': No such file or directory$"
+            not_found_pattern = "^stat: `.*': No such file or directory$"
             not_found_re = re.compile(not_found_pattern)
             for line in stderr.split('\n'):
                 if not_found_re.match(line):
@@ -85,15 +87,33 @@ class HdfsClient(object):
 
     def remove(self, path, recursive=True, skip_trash=False):
         if recursive:
-            if use_cdh4_syntax():
-                cmd = ['hadoop', 'fs', '-rm', '-r']
-            else:
-                cmd = ['hadoop', 'fs', '-rmr']
+            cmd = ['hadoop', 'fs', '-rm', '-r']
         else:
             cmd = ['hadoop', 'fs', '-rm']
+
         if skip_trash:
-            cmd.append("-skipTrash")
-        cmd.append(path)
+            cmd = cmd + ['-skipTrash']
+
+        cmd = cmd + [path]
+        call_check(cmd)
+
+    def chmod(self, path, permissions, recursive=False):
+        if recursive:
+            cmd = ['hadoop', 'fs', '-chmod', '-R', permissions, path]
+        else:
+            cmd = ['hadoop', 'fs', '-chmod', permissions, path]
+        call_check(cmd)
+
+    def chown(self, path, owner, group, recursive=False):
+        if owner is None:
+            owner = ''
+        if group is None:
+            group = ''
+        ownership = "%s:%s" % (owner, group)
+        if recursive:
+            cmd = ['hadoop', 'fs', '-chown', '-R', ownership, path]
+        else:
+            cmd = ['hadoop', 'fs', '-chown', ownership, path]
         call_check(cmd)
 
     def count(self, path):
@@ -120,14 +140,23 @@ class HdfsClient(object):
         call_check(cmd)
 
     def mkdir(self, path):
-        call_check(['hadoop', 'fs', '-mkdir', path])
+        try:
+            call_check(['hadoop', 'fs', '-mkdir', path])
+        except HDFSCliError, ex:
+            if "File exists" in ex.stderr:
+                raise FileAlreadyExists(ex.stderr)
+            else:
+                raise
 
     def listdir(self, path, ignore_directories=False, ignore_files=False,
-                include_size=False, include_type=False, include_time=False):
+                include_size=False, include_type=False, include_time=False, recursive=False):
         if not path:
             path = "."  # default to current/home catalog
 
-        cmd = ['hadoop', 'fs', '-ls', path]
+        if recursive:
+            cmd = ['hadoop', 'fs', '-ls', '-R', path]
+        else:
+            cmd = ['hadoop', 'fs', '-ls', path]
         proc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
         lines = proc.stdout
 
@@ -161,7 +190,42 @@ class HdfsClient(object):
             else:
                 yield file
 
-client = HdfsClient()
+class HdfsClientCdh3(HdfsClient):
+    """This client uses CDH3 syntax for file system commands"""
+    def remove(self, path, recursive=True, skip_trash=False):
+        if recursive:
+            cmd = ['hadoop', 'fs', '-rmr']
+        else:
+            cmd = ['hadoop', 'fs', '-rm']
+
+        if skip_trash:
+            cmd = cmd + ['-skipTrash']
+
+        cmd = cmd + [path]
+        call_check(cmd)
+
+class HdfsClientApache1(HdfsClientCdh3):
+    """This client uses Apache 1.x syntax for file system commands,
+    which are similar to CDH3 except for the file existence check"""
+    def exists(self, path):
+        cmd = ['hadoop', 'fs', '-test', '-e', path]
+        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        stdout, stderr = p.communicate()
+        if p.returncode == 0:
+            return True
+        elif p.returncode == 1:
+            return False
+        else:
+            raise HDFSCliError(cmd, p.returncode, stdout, stderr)
+
+if get_hdfs_syntax() == "cdh4":
+    client = HdfsClient()
+elif get_hdfs_syntax() == "cdh3":
+    client = HdfsClientCdh3()
+elif get_hdfs_syntax() == "apache1":
+    client = HdfsClientApache1()
+else:
+    raise Exception("Error: Unknown version specified in Hadoop version configuration parameter")
 
 exists = client.exists
 rename = client.rename
@@ -191,7 +255,7 @@ class HdfsAtomicWritePipe(luigi.format.OutputPipeProcessWrapper):
         self.path = path
         self.tmppath = tmppath(self.path)
         tmpdir = os.path.dirname(self.tmppath)
-        if use_cdh4_syntax():
+        if get_hdfs_syntax() == "cdh4":
             if subprocess.Popen(['hadoop', 'fs', '-mkdir', '-p', tmpdir]).wait():
                 raise RuntimeError("Could not create directory: %s" % tmpdir)
         else:
@@ -248,12 +312,14 @@ class PlainDir(luigi.format.Format):
         return HdfsAtomicWriteDirPipe(path)
 
 
-class HdfsTarget(luigi.Target):
+class HdfsTarget(FileSystemTarget):
+    fs = client  # underlying file system
+
     def __init__(self, path=None, format=Plain, is_tmp=False):
         if path is None:
             assert is_tmp
             path = tmppath()
-        self.path = path
+        super(HdfsTarget, self).__init__(path)
         self.format = format
         self.is_tmp = is_tmp
         (scheme, netloc, path, query, fragment) = urlparse.urlsplit(path)
@@ -279,9 +345,6 @@ in luigi. Use target.path instead", stacklevel=2)
 in luigi. Use target.path instead", stacklevel=2)
         return self.path
 
-    def exists(self):
-        return exists(self.path)
-
     def glob_exists(self, expected_files):
         ls = list(listdir(self.path))
         if len(ls) == expected_files:
@@ -303,8 +366,8 @@ in luigi. Use target.path instead", stacklevel=2)
             except NotImplementedError:
                 return self.format.pipe_writer(HdfsAtomicWritePipe(self.path))
 
-    def remove(self):
-        remove(self.path)
+    def remove(self, skip_trash=False):
+        remove(self.path, skip_trash=skip_trash)
 
     def rename(self, path, fail_if_exists=False):
         # rename does not change self.path, so be careful with assumptions
